package com.pan_american.android.ui.group_invite_official.invite_data

import android.Manifest
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.text.Editable
import android.text.TextWatcher
import android.util.Base64
import android.view.Gravity
import android.view.View
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.PopupWindow
import android.widget.TextView
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import androidx.core.content.res.ResourcesCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.pan_american.android.OASystem
import com.pan_american.android.R
import com.pan_american.android.base.BaseActivity
import com.pan_american.android.base.BaseRequest
import com.pan_american.android.base.BaseResponse
import com.pan_american.android.base.CardAdapter
import com.pan_american.android.base.CustomAlertDialog
import com.pan_american.android.base.ListAdapter
import com.pan_american.android.data.model.common.entity.Selector
import com.pan_american.android.data.model.common.network.SelectorResponse
import com.pan_american.android.data.model.group_invite_official.invite_data.network.InviteDataDetailRequest
import com.pan_american.android.data.model.group_invite_official.invite_data.network.InviteDataDetailResponse
import com.pan_american.android.data.model.group_invite_official.invite_data.network.UpdateInviteDetailRequest
import com.pan_american.android.data.model.resource_management.market_customer.network.BusinessCardInfoRequest
import com.pan_american.android.data.model.resource_management.market_customer.network.BusinessCardInfoResponse
import com.pan_american.android.data.network.APIService
import com.pan_american.android.data.network.ServiceCreator
import com.pan_american.android.databinding.ActivityAddInviteDataBinding
import com.pan_american.android.databinding.LayoutTitleBinding
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.ByteArrayOutputStream
import java.io.File

class AddInviteDataActivity : BaseActivity<ActivityAddInviteDataBinding>() {

    private var fromList = false

    private var dataId = 0

    private lateinit var titleBinding: LayoutTitleBinding

    private val apiService = ServiceCreator.create<APIService>()

    private val groupList = ArrayList<Selector>()

    private val groupSelected = ArrayList<Selector>()

    private lateinit var adapter: CardAdapter<Selector>

    private var updateInviteDataDetailRequest = UpdateInviteDetailRequest()

    //监听拍照事件回调
    private lateinit var result: ActivityResultLauncher<Intent>

    //名片识别，图片Uri
    private lateinit var imageUri: Uri

    //输出文件路径
    private lateinit var outputImage: File

    //识别后手机号可能存在多个
    private var telephoneOCR = ""

    //识别后电话号码可能存在多个
    private var phoneOCR = ""

    //识别后地址可能存在多行
    private var locationOCR = ""

    override fun getViewBinding() = ActivityAddInviteDataBinding.inflate(layoutInflater)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        intent.apply {
            fromList = getBooleanExtra("from_list", false)
            dataId = getIntExtra("data_id", 0)
        }

        result = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            if (it.resultCode == Activity.RESULT_OK) {

                telephoneOCR = ""
                phoneOCR = ""
                locationOCR = ""

                //调用OCR接口进行名片识别
                getBusinessCardInfo()
            }
        }

        initTitle()

        getGroupNameList()
    }

    override fun initTitle() {
        titleBinding = LayoutTitleBinding.bind(binding.root).apply {
            if (fromList) {
                titleText.text = resources.getString(R.string.invite_data_detail)
            } else {
                titleText.text = resources.getString(R.string.add_invite_data)

                rightTextField.visibility = View.VISIBLE
            }

            backButton.setOnClickListener {
                back()
            }
        }

        titleBinding.rightTextField.setOnClickListener {
            if (ContextCompat.checkSelfPermission(
                    OASystem.context, Manifest.permission.CAMERA
                ) != PackageManager.PERMISSION_GRANTED
            ) {
                ActivityCompat.requestPermissions(
                    this, arrayOf(Manifest.permission.CAMERA), OASystem.CAMERA_REQUEST_CODE
                )
            } else {
                startGetPhoto()
            }
        }
    }

    override fun initViews() {
        if (fromList) {

            getInviteDataDetail()

            binding.commit.text = resources.getString(R.string.update)

            if (!OASystem.authorization(OASystem.INVITE_DATA, OASystem.EDIT)) {
                binding.apply {
                    country.isEnabled = false
                    city.isEnabled = false
                    inviteeName.isEnabled = false
                    inviteeWebsite.isEnabled = false
                    industry.isEnabled = false
                    inviteeLocation.isEnabled(false)
                    inviteeInfo.isEnabled(false)
                    inviteContacts.isEnabled = false
                    contactsJob.isEnabled = false
                    contactsPhone.isEnabled = false
                    contactsEmail.isEnabled = false
                    wechat.isEnabled = false
                    faceBook.isEnabled = false
                    instagram.isEnabled = false
                    contactsFax.isEnabled = false
                    add.visibility = View.GONE
                    otherInfo.isEnabled(false)

                    commit.text = resources.getString(R.string.back)
                }
            }

        } else {

            binding.commit.text = resources.getString(R.string.add)

        }

        initEvents()
    }

    override fun initEvents() {
        binding.add.setOnClickListener {
            val selectorList = ArrayList<Selector>()

            for (item in groupList) {
                selectorList.add(item)
            }

            val popView = View.inflate(OASystem.context, R.layout.popup_selector, null)
            popupWindow = PopupWindow(popView, RecyclerView.LayoutParams.MATCH_PARENT, RecyclerView.LayoutParams.WRAP_CONTENT)

            showPopupWindow {

                val searchView: LinearLayout = popView.findViewById(R.id.search_view)
                searchView.visibility = View.VISIBLE

                val searchText: EditText = popView.findViewById(R.id.search_text)

                selector = popView.findViewById(R.id.selector_list)

                val layoutManager = LinearLayoutManager(OASystem.context)
                selector.layoutManager = layoutManager

                val adapter = ListAdapter.Builder<Selector>().apply {
                    setData(selectorList)
                    setLayoutId(R.layout.item_selector)
                    addBindView { itemView, data ->
                        itemView.findViewById<TextView>(R.id.selector_item_name).apply {
                            text = data.groupName

                            if (groupSelected.contains(data)) {
                                setTextColor(ResourcesCompat.getColor(resources, R.color.text_color_blue, null))
                            } else {
                                setTextColor(ResourcesCompat.getColor(resources, R.color.text_color, null))
                            }
                        }
                    }
                }.create()

                selector.adapter = adapter

                adapter.onRecyclerViewItemClick =
                    object : ListAdapter.OnRecyclerViewItemClick<Selector> {
                        override fun onItemClick(position: Int) {
                            selectorList[position].apply {

                                if (groupSelected.contains(this)) {
                                    showMessage(resources.getString(R.string.association_group_selected_hint))
                                    return
                                } else {
                                    groupSelected.add(this)
                                    initGroupNameList()
                                }

                                popupWindow.dismiss()
                            }
                        }
                    }

                searchText.addTextChangedListener(object : TextWatcher {
                    override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

                    }

                    override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

                    }

                    override fun afterTextChanged(editable: Editable?) {

                        adapter.notifyItemRangeRemoved(0, selectorList.size)

                        selectorList.clear()

                        if (editable.isNullOrBlank()) {

                            for (item in groupList) {
                                selectorList.add(item)
                            }

                        } else {
                            for (i in 0 until groupList.size) {
                                groupList[i].apply {
                                    if (groupName.contains(editable)) {
                                        selectorList.add(this)
                                    }
                                }
                            }
                        }

                        if (selectorList.size == 0) {
                            showMessage(resources.getString(R.string.no_data))
                        } else {
                            for (i in 0 until selectorList.size) {
                                adapter.notifyItemInserted(i)
                            }
                        }
                    }
                })

                popupWindow.showAtLocation(binding.root, Gravity.BOTTOM, 0, 0)
            }
        }

        binding.commit.setOnClickListener {
            if (!OASystem.authorization(OASystem.INVITE_DATA, OASystem.EDIT)) {
                back()
                return@setOnClickListener
            }

            if (binding.country.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.invitee_country_input_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.country = binding.country.text.toString()
            }

            if (binding.city.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.invitee_city_input_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.city = binding.city.text.toString()
            }

            if (binding.inviteeName.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.invitee_name_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.unitName = binding.inviteeName.text.toString()
            }

            if (binding.inviteeWebsite.text.isNotBlank()) {
                updateInviteDataDetailRequest.unitWeb = binding.inviteeWebsite.text.toString()
            }

            if (binding.industry.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.invitee_industry_input_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.field = binding.industry.text.toString()
            }

            if (binding.inviteeLocation.getText().isBlank()) {
                showMessage(resources.getString(R.string.invitee_location_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.address = binding.inviteeLocation.getText()
            }

            if (binding.inviteeInfo.getText().isNotBlank()) {
                updateInviteDataDetailRequest.unitInfo = binding.inviteeInfo.getText()
            }

            if (binding.inviteContacts.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.invitee_contacts_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.contact = binding.inviteContacts.text.toString()
            }

            if (binding.contactsJob.text.isNotBlank()) {
                updateInviteDataDetailRequest.job = binding.contactsJob.text.toString()
            }

            if (binding.contactsPhone.text.isNullOrBlank()) {
                showMessage(resources.getString(R.string.contacts_tel_hint))
                return@setOnClickListener
            } else {
                updateInviteDataDetailRequest.tel = binding.contactsPhone.text.toString()
            }

            if (binding.contactsEmail.text.isNotBlank()) {
                updateInviteDataDetailRequest.email = binding.contactsEmail.text.toString()
            }

            if (binding.wechat.text.isNotBlank()) {
                updateInviteDataDetailRequest.weChat = binding.wechat.text.toString()
            }

            if (binding.faceBook.text.isNotBlank()) {
                updateInviteDataDetailRequest.faceBook = binding.faceBook.text.toString()
            }

            if (binding.instagram.text.isNotBlank()) {
                updateInviteDataDetailRequest.ins = binding.instagram.text.toString()
            }

            if (binding.contactsFax.text.isNotBlank()) {
                updateInviteDataDetailRequest.fax = binding.contactsFax.text.toString()
            }

            if (groupSelected.size > 0) {
                var i = 0

                var groupIdStr = ""

                while (i < groupSelected.size) {
                    groupIdStr += groupSelected[i].id

                    i++

                    if (i < groupSelected.size) {
                        groupIdStr += ","
                    }
                }

                updateInviteDataDetailRequest.delegation = groupIdStr
            }

            if (binding.otherInfo.getText().isNotBlank()) {
                updateInviteDataDetailRequest.remark = binding.otherInfo.getText()
            }

            if (fromList) {
                updateInviteDataDetailRequest.status = 2
            } else {
                updateInviteDataDetailRequest.status = 1
            }

            updateInviteData()
        }
    }

    private fun getGroupNameList() {
        apiService.getGroupNameList(BaseRequest()).enqueue(object : Callback<SelectorResponse> {
            override fun onResponse(
                call: Call<SelectorResponse>,
                response: Response<SelectorResponse>
            ) {
                val listResponse = response.body()

                if (listResponse != null) {
                    if (listResponse.code == 200) {

                        for (item in listResponse.data) {
                            groupList.add(item)
                        }

                        initViews()

                    } else {
                        showMessage(listResponse.msg)
                    }
                }
            }

            override fun onFailure(call: Call<SelectorResponse>, t: Throwable) {
                showMessage(resources.getString(R.string.network_error))
            }
        })
    }

    private fun getInviteDataDetail() {
        apiService.getInviteDataDetailById(InviteDataDetailRequest(dataId)).enqueue(object : Callback<InviteDataDetailResponse> {
            override fun onResponse(
                call: Call<InviteDataDetailResponse>,
                response: Response<InviteDataDetailResponse>
            ) {
                val detailResponse = response.body()

                if (detailResponse != null) {
                    if (detailResponse.code == 200) {

                        detailResponse.data.apply {

                            updateInviteDataDetailRequest.id = id
                            updateInviteDataDetailRequest.createUserId = createUserId

                            binding.country.setText(country)
                            binding.city.setText(city)
                            binding.inviteeName.setText(unitName)
                            binding.inviteeWebsite.setText(unitWeb)
                            binding.industry.setText(field)
                            binding.inviteeLocation.setText(address)
                            binding.inviteeInfo.setText(unitInfo)
                            binding.inviteContacts.setText(contact)
                            binding.contactsJob.setText(job)
                            binding.contactsPhone.setText(tel)
                            binding.contactsEmail.setText(email)
                            binding.wechat.setText(weChat)
                            binding.faceBook.setText(faceBook)
                            binding.instagram.setText(ins)

                            if (delegation.isNotBlank()) {
                                val groupIds = delegation.split(",")

                                for (checkedItem in groupIds) {
                                    for (item in groupList) {
                                        if (checkedItem.toInt() == item.id) {
                                            groupSelected.add(item)
                                        }
                                    }
                                }

                                initGroupNameList()
                            }

                            binding.contactsFax.setText(fax)
                            binding.otherInfo.setText(remark)
                        }

                    } else {
                        showMessage(detailResponse.msg)
                    }
                }
            }

            override fun onFailure(call: Call<InviteDataDetailResponse>, t: Throwable) {
                showMessage(resources.getString(R.string.network_error))
            }
        })
    }

    private fun initGroupNameList() {
        val layoutManager = LinearLayoutManager(OASystem.context)
        binding.groupList.layoutManager = layoutManager

        adapter = CardAdapter.Builder<Selector>().apply {
            setData(groupSelected)
            setLayoutId(R.layout.item_delete_button_selector)
            setCanDelete(OASystem.authorization(OASystem.INVITE_DATA, OASystem.EDIT))
            addBindView { itemView, data ->
                itemView.findViewById<TextView>(R.id.left_text).text = data.groupName
            }
        }.create()

        binding.groupList.adapter = adapter

        adapter.onRecyclerViewItemClick = object : CardAdapter.OnRecyclerViewItemClick<Selector> {
            override fun onItemClick(position: Int) {

            }

            override fun onItemDelete(position: Int) {
                CustomAlertDialog.Builder(OASystem.context).apply {
                    setTitle(resources.getString(R.string.alert))
                    setMessage(String.format(resources.getString(R.string.delete_alert_text), groupSelected[position].groupName))
                    setNegativeButtonAndListener(resources.getString(R.string.cancel)) { dialog, _ ->
                        dialog.dismiss()
                    }
                    setPositiveButtonAndListener(resources.getString(R.string.confirm)) { _, _ ->
                        groupSelected.removeAt(position)
                        binding.groupList.adapter!!.notifyItemRemoved(position)
                        binding.groupList.adapter!!.notifyItemRangeChanged(position, groupSelected.size)
                    }
                }.show()
            }
        }
    }

    private fun updateInviteData() {

        binding.commit.isEnabled = false

        binding.commit.visibility = View.GONE

        binding.updateLoading.visibility = View.VISIBLE

        apiService.updateInviteData(updateInviteDataDetailRequest).enqueue(object : Callback<BaseResponse> {
            override fun onResponse(call: Call<BaseResponse>, response: Response<BaseResponse>) {

                val updateResponse = response.body()

                if (updateResponse != null) {
                    if (updateResponse.code == 200) {

                        if (fromList) {
                            showMessage(resources.getString(R.string.update_success))
                        } else {
                            showMessage(resources.getString(R.string.insert_success))
                        }

                        OASystem.needRefresh = true

                        back()

                    } else {
                        showMessage(updateResponse.msg)

                        binding.commit.isEnabled = true

                        binding.commit.visibility = View.VISIBLE

                        binding.updateLoading.visibility = View.GONE
                    }
                }

            }

            override fun onFailure(call: Call<BaseResponse>, t: Throwable) {

                binding.commit.isEnabled = true

                binding.commit.visibility = View.VISIBLE

                binding.updateLoading.visibility = View.GONE

                showMessage(resources.getString(R.string.network_error))
            }
        })
    }

    private fun startGetPhoto() {
        //创建File对象，用于存储拍照后的图片
        outputImage = File(externalCacheDir, "output_image.jpg")

        if (outputImage.exists()) {
            outputImage.delete()
        }

        outputImage.createNewFile()
        imageUri =
            FileProvider.getUriForFile(this, "com.pan_american.android.file_provider", outputImage)

        //启动相机程序
        val intent = Intent("android.media.action.IMAGE_CAPTURE")
        intent.putExtra(MediaStore.EXTRA_OUTPUT, imageUri)
        result.launch(intent)
    }

    private fun getBusinessCardInfo() {

        var base64Adjusted = false

        var bitmapSampleSize = 2

        var bitmapQuality = 100

        val byteArrayOutputStream = ByteArrayOutputStream()

        var transferBase64: String

        do {
            val options = BitmapFactory.Options()
            options.inSampleSize = bitmapSampleSize

            val bitmap =
                BitmapFactory.decodeStream(contentResolver.openInputStream(imageUri), null, options)

            bitmap!!.compress(Bitmap.CompressFormat.JPEG, bitmapQuality, byteArrayOutputStream)

            transferBase64 =
                Base64.encodeToString(byteArrayOutputStream.toByteArray(), Base64.NO_WRAP)

            if (transferBase64.length < 1024 * 1024 * 2) {
                if (transferBase64.length < 1024 * 1024) {
                    base64Adjusted = true
                    byteArrayOutputStream.close()
                } else {
                    bitmapQuality -= 10
                }
            } else {
                bitmapSampleSize *= 2
            }
            byteArrayOutputStream.reset()
            bitmap.recycle()
        } while (!base64Adjusted)

        apiService.getBusinessCardInfo(BusinessCardInfoRequest(transferBase64))
            .enqueue(object : Callback<BusinessCardInfoResponse> {

                override fun onResponse(
                    call: Call<BusinessCardInfoResponse>,
                    response: Response<BusinessCardInfoResponse>
                ) {
                    val infoResponse = response.body()

                    if (infoResponse != null) {

                        if (infoResponse.code == 200) {

                            infoResponse.data.businessCardInfos.forEach {
                                when (it.name) {
                                    "姓名" -> {
                                        binding.inviteContacts.setText(it.value)
                                    }

                                    "手机" -> {
                                        telephoneOCR += if (telephoneOCR.isEmpty()) {
                                            it.value
                                        } else {
                                            " ,${it.value}"
                                        }
                                    }

                                    "电话" -> {
                                        phoneOCR += if (phoneOCR.isEmpty()) {
                                            it.value
                                        } else {
                                            " ,${it.value}"
                                        }
                                    }

                                    "邮箱" -> {
                                        binding.contactsEmail.setText(it.value)
                                    }

                                    "职位" -> {
                                        binding.contactsJob.setText(it.value)
                                    }

                                    "公司" -> {
                                        binding.inviteeName.setText(it.value)
                                    }

                                    "地址" -> {
                                        locationOCR += it.value
                                    }
                                }
                            }

                            if (telephoneOCR.isNotBlank()) {
                                binding.contactsPhone.setText(telephoneOCR)
                            } else {
                                binding.contactsPhone.setText(phoneOCR)
                            }

                            binding.inviteeLocation.setText(locationOCR)

                            outputImage.delete()

                        } else {
                            showMessage(infoResponse.msg)
                        }
                    }
                }

                override fun onFailure(call: Call<BusinessCardInfoResponse>, t: Throwable) {
                    showMessage("网络错误，请检查网络设置")
                }
            })
    }
}